@@ -672,8 +672,7 @@ class PRST1(USBMS):
                                 'WHERE content_id = ? AND collection_id = ? ')
                         t = (bookId, db_collections[collection],)
                         cursor.execute(query, t)
-                        debug_print('Deleted Book From Collection: ' + bookPath
-                                + ' -> ' + collection)
+                        debug_print('Deleted Book From Collection: ' + bookPath + ' -> ' + collection)
 
                 db_collections[collection] = None
 
@@ -761,7 +760,7 @@ class PRST1(USBMS):
         if not os.path.exists(thumbnail_dir_path):
             os.makedirs(thumbnail_dir_path)
 
-        with open(thumbnail_file_path, 'wb') as f:
+        with lopen(thumbnail_file_path, 'wb') as f:
             f.write(book.thumbnail[-1])
             fsync(f)
 
