@@ -17,12 +17,13 @@ from PyQt5.QtWebKitWidgets import QWebView
 
 from calibre import fit_image
 from calibre.gui2.dnd import (dnd_has_image, dnd_get_image, dnd_get_files,
-    IMAGE_EXTENSIONS, dnd_has_extension)
+    dnd_has_extension, image_extensions)
 from calibre.ebooks import BOOK_EXTENSIONS
 from calibre.ebooks.metadata.book.base import (field_metadata, Metadata)
 from calibre.ebooks.metadata.book.render import mi_to_html
 from calibre.gui2 import (config, open_url, pixmap_to_data, gprefs, rating_font, NO_URL_FORMATTING)
 from calibre.utils.config import tweaks
+from calibre.utils.img import image_from_x, blend_image
 from calibre.utils.localization import is_rtl
 
 _css = None
@@ -289,7 +290,10 @@ class CoverView(QWidget):  # {{{
         target = QRect(x, y, width, height)
         p = QPainter(self)
         p.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
-        dpr = self.devicePixelRatio()
+        try:
+            dpr = self.devicePixelRatioF()
+        except AttributeError:
+            dpr = self.devicePixelRatio()
         spmap = self.pixmap.scaled(target.size() * dpr, Qt.KeepAspectRatio, Qt.SmoothTransformation)
         spmap.setDevicePixelRatio(dpr)
         p.drawPixmap(target, spmap)
@@ -365,6 +369,8 @@ class CoverView(QWidget):  # {{{
             pmap.loadFromData(cdata)
         if pmap.isNull():
             return
+        if pmap.hasAlphaChannel():
+            pmap = QPixmap.fromImage(blend_image(image_from_x(pmap)))
         self.pixmap = pmap
         self.do_layout()
         self.update()
@@ -636,11 +642,10 @@ class BookDetails(QWidget):  # {{{
     open_fmt_with = pyqtSignal(int, object, object)
 
     # Drag 'n drop {{{
-    DROPABBLE_EXTENSIONS = IMAGE_EXTENSIONS+BOOK_EXTENSIONS
 
     def dragEnterEvent(self, event):
         md = event.mimeData()
-        if dnd_has_extension(md, self.DROPABBLE_EXTENSIONS, allow_all_extensions=True) or \
+        if dnd_has_extension(md, image_extensions() + BOOK_EXTENSIONS, allow_all_extensions=True) or \
                 dnd_has_image(md):
             event.acceptProposedAction()
 
@@ -648,7 +653,8 @@ class BookDetails(QWidget):  # {{{
         event.setDropAction(Qt.CopyAction)
         md = event.mimeData()
 
-        x, y = dnd_get_image(md)
+        image_exts = set(image_extensions()) - set(tweaks['cover_drop_exclude'])
+        x, y = dnd_get_image(md, image_exts)
         if x is not None:
             # We have an image, set cover
             event.accept()
@@ -663,7 +669,7 @@ class BookDetails(QWidget):  # {{{
                 return
 
         # Now look for ebook files
-        urls, filenames = dnd_get_files(md, BOOK_EXTENSIONS, allow_all_extensions=True)
+        urls, filenames = dnd_get_files(md, BOOK_EXTENSIONS, allow_all_extensions=True, filter_exts=image_exts)
         if not urls:
             # Nothing found
             return
