@@ -17,7 +17,7 @@ from PyQt5.Qt import (
     QColorDialog, QTimer, pyqtSignal)
 
 from calibre import prepare_string_for_xml
-from calibre.gui2.tweak_book import tprefs, TOP
+from calibre.gui2.tweak_book import tprefs, TOP, current_container
 from calibre.gui2.tweak_book.completion.popup import CompletionPopup
 from calibre.gui2.tweak_book.editor import (
     SYNTAX_PROPERTY, SPELL_PROPERTY, SPELL_LOCALE_PROPERTY, store_locale, LINK_PROPERTY)
@@ -313,7 +313,12 @@ class TextEdit(PlainTextEdit):
             raw, count = pat.subn(template, raw)
             if repl_is_func:
                 from calibre.gui2.tweak_book.search import show_function_debug_output
-                template.end()
+                if getattr(template.func, 'append_final_output_to_marked', False):
+                    retval = template.end()
+                    if retval:
+                        raw += unicode(retval)
+                else:
+                    template.end()
                 show_function_debug_output(template)
             if count > 0:
                 start_pos = min(c.anchor(), c.position())
@@ -327,6 +332,21 @@ class TextEdit(PlainTextEdit):
         from calibre.gui2.tweak_book.editor.comments import smart_comment
         smart_comment(self, self.syntax)
 
+    def sort_css(self):
+        from calibre.gui2.dialogs.confirm_delete import confirm
+        if confirm(_('Sorting CSS rules can in rare cases change the effective styles applied to the book.'
+                     ' Are you sure you want to proceed?'), 'edit-book-confirm-sort-css', parent=self, config_set=tprefs):
+            c = self.textCursor()
+            c.beginEditBlock()
+            c.movePosition(c.Start), c.movePosition(c.End, c.KeepAnchor)
+            text = unicode(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+            from calibre.ebooks.oeb.polish.css import sort_sheet
+            text = sort_sheet(current_container(), text).cssText
+            c.insertText(text)
+            c.movePosition(c.Start)
+            c.endEditBlock()
+            self.setTextCursor(c)
+
     def find(self, pat, wrap=False, marked=False, complete=False, save_match=None):
         if marked:
             return self.find_in_marked(pat, wrap=wrap, save_match=save_match)
@@ -368,6 +388,40 @@ class TextEdit(PlainTextEdit):
             self.saved_matches[save_match] = (pat, m)
         return True
 
+    def find_text(self, pat, wrap=False, complete=False):
+        reverse = pat.flags & regex.REVERSE
+        c = self.textCursor()
+        c.clearSelection()
+        if complete:
+            # Search the entire text
+            c.movePosition(c.End if reverse else c.Start)
+        pos = c.Start if reverse else c.End
+        if wrap and not complete:
+            pos = c.End if reverse else c.Start
+        c.movePosition(pos, c.KeepAnchor)
+        if hasattr(self.smarts, 'find_text'):
+            self.highlighter.join()
+            found, start, end = self.smarts.find_text(pat, c)
+            if not found:
+                return False
+        else:
+            raw = unicode(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+            m = pat.search(raw)
+            if m is None:
+                return False
+            start, end = m.span()
+            if start == end:
+                return False
+        if reverse:
+            start, end = end, start
+        c.clearSelection()
+        c.setPosition(start)
+        c.setPosition(end, c.KeepAnchor)
+        self.setTextCursor(c)
+        # Center search result on screen
+        self.centerCursor()
+        return True
+
     def find_spell_word(self, original_words, lang, from_cursor=True, center_on_cursor=True):
         c = self.textCursor()
         c.setPosition(c.position())
@@ -391,10 +445,13 @@ class TextEdit(PlainTextEdit):
             c.setPosition(c.anchor() + idx)
             c.setPosition(c.position() + string_length(word), c.KeepAnchor)
             if self.smarts.verify_for_spellcheck(c, self.highlighter):
-                self.setTextCursor(c)
-                if center_on_cursor:
-                    self.centerCursor()
-                return True
+                self.highlighter.join()  # Ensure highlighting is finished
+                locale = self.spellcheck_locale_for_cursor(c)
+                if not lang or not locale or (locale and lang == locale.langcode):
+                    self.setTextCursor(c)
+                    if center_on_cursor:
+                        self.centerCursor()
+                    return True
             c.setPosition(c.position())
             c.movePosition(c.End, c.KeepAnchor)
 
@@ -563,7 +620,7 @@ class TextEdit(PlainTextEdit):
             formats = self.highlighter.parse_single_block(c.block())[0]
         pos = c.positionInBlock()
         for r in formats:
-            if r.start <= pos < r.start + r.length and r.format.property(SPELL_PROPERTY):
+            if r.start <= pos <= r.start + r.length and r.format.property(SPELL_PROPERTY):
                 return r.format.property(SPELL_LOCALE_PROPERTY)
 
     def recheck_word(self, word, locale):
@@ -583,7 +640,7 @@ class TextEdit(PlainTextEdit):
             return
         pos = cursor.positionInBlock()
         for r in cursor.block().layout().additionalFormats():
-            if r.start <= pos < r.start + r.length and r.format.property(SYNTAX_PROPERTY):
+            if r.start <= pos <= r.start + r.length and r.format.property(SYNTAX_PROPERTY):
                 return r
 
     def syntax_format_for_cursor(self, cursor):
